﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.EntityFrameworkCore;
using InventarioPaldaca.Models.ViewModels;
using InventarioPaldaca.Models.Inventario;
using System.Linq;
using System.Threading.Tasks;
using System.Collections.Generic;
using InventarioPaldaca.Models;
using System.Diagnostics;
using static Microsoft.EntityFrameworkCore.DbLoggerCategory;

namespace InventarioPaldaca.Controllers
{
    public class ActivoController : Controller
    {
        private readonly InventaryPaldacaContext _context;

        public ActivoController(InventaryPaldacaContext context)
        {
            _context = context;
        }
        public async Task<IActionResult> Index()
        {
            ViewData["Usuarios"] = new SelectList(_context.Usuarios
                                             .Select(u => new {
                                                 u.UsuarioId,
                                                 NombreCompleto = u.UsuarioNombre + " " + u.UsuarioApellido
                                             }), "UsuarioId", "NombreCompleto");

            // Obtener todos los activos incluyendo sus relaciones
            var Activos = await _context.Activos
                               .Include(u => u.Usuario)
                               .Include(u => u.Ubicacion)
                               .Include(u => u.Categoria)
                               .ThenInclude(c => c.CategoriaMaster)
                               .ToListAsync();

           
       

            // Calcular el total de activos
            int totalActivos = Activos?.Count ?? 0;

            // Obtener todas las categorías disponibles
            var categorias = await _context.Categoria.ToListAsync();

            // Obtener todas las ubicaciones disponibles
            var ubicaciones = await _context.Ubicacions.ToListAsync();
           

            int totalActivosDañados = Activos?.Count(a => a.Funcionabilidad == false) ?? 0;

            // Obtener todas las categorías maestras disponibles
            var categoriasMaster = await _context.CategoriaMasters.ToListAsync();

            // Contar los activos por categoría
            var activosPorCategoria = Activos?
                .GroupBy(a => a.Categoria.CategoriaNombre)
                .ToDictionary(g => g.Key, g => g.Count())
                ?? new Dictionary<string, int>();

            // Contar los activos por ubicación
            var activosPorUbicacion = Activos?
                .GroupBy(a => a.Ubicacion.UbicacionNombre)
                .ToDictionary(g => g.Key, g => g.Count())
                ?? new Dictionary<string, int>();

            // Crear el ViewModel
            var model = new ListaActivosViewModel
            {
                TotalActivos = totalActivos,
                ActivosPorCategoria = activosPorCategoria,
                ActivosPorUbicacion = activosPorUbicacion,
                CategoriasMaster = categoriasMaster ?? new List<CategoriaMaster>(),
                ListaActivos = Activos ?? new List<Activo>(),
                Categorias = categorias ?? new List<Categorium>(),
                Ubicaciones = ubicaciones ?? new List<Ubicacion>(),
                TotalActivosDañados = totalActivosDañados
            };

            return View(model);
        }
        public IActionResult ActivosDañados()
        {
            var activosDañados = _context.Activos
                                          .Include(a => a.Usuario)
                                          .Include(a => a.Categoria)
                                          .Include(a => a.Ubicacion)
                                         .Where(a => a.Funcionabilidad == false)
                                          .ToList();

            var model = new ListaActivosViewModel
            {
                ListaActivos = activosDañados
            };

            return View("ActivosDañados", model);
        }
        public IActionResult Create()
        {
            ViewData["Categoria"] = new SelectList(_context.Categoria, "CategoriaId", "CategoriaNombre");
            ViewData["Ubicacion"] = new SelectList(_context.Ubicacions, "UbicacionId", "UbicacionNombre");
            ViewData["Usuario"] = new SelectList(_context.Usuarios
                                                        .Select(u => new {
                                                           u.UsuarioId,
                                                           NombreCompleto = u.UsuarioNombre + " " + u.UsuarioApellido
                                                         }), "UsuarioId", "NombreCompleto");
            return View();
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<IActionResult> Create(ActivoViewModel model)
        {
            if (ModelState.IsValid)
            {
                var activo = new Activo()
                {
                    Marca = model.Marca,
                    Modelo = model.Modelo,
                    NumeroSerial = model.NumeroSerial,
                    Funcionabilidad = model.Funcionabilidad,
                    Observaciones = model.Observaciones,
                    CodigoInventario = model.CodigoInventario,
                    CategoriaId = model.CategoriaId,
                    UbicacionId = model.UbicacionId,
                    UsuarioId = model.UsuarioId,
                    Adquisicion = model.AñoAdquirido
                };

                try
                {
                    _context.Add(activo);
                    await _context.SaveChangesAsync();
                    return RedirectToAction(nameof(Index));
                }
                catch (Exception ex)
                {
                    // Manejar excepción o registrar el error
                    Console.WriteLine(ex.Message);
                    ModelState.AddModelError("", "No se pudo guardar el activo. Intente nuevamente.");
                }
            }
            else
            {
                foreach (var error in ModelState.Values.SelectMany(v => v.Errors))
                {
                    Console.WriteLine(error.ErrorMessage); 
                }
            }
            ViewData["Categoria"] = new SelectList(_context.Categoria, "CategoriaId", "CategoriaNombre", model.CategoriaId);
            ViewData["Ubicacion"] = new SelectList(_context.Ubicacions, "UbicacionId", "UbicacionNombre", model.UbicacionId);
            ViewData["Usuario"] = new SelectList(_context.Usuarios
                                                                  .Select(u => new {
                                                                      u.UsuarioId,
                                                                      NombreCompleto = u.UsuarioNombre + " " + u.UsuarioApellido
                                                                  }), "UsuarioId", "NombreCompleto", model.UsuarioId);
            return View(model);
        }
        

        [HttpPost]
        public async Task<IActionResult> Delete(int id)
        {
            try
            {
                var activo = await _context.Activos.FindAsync(id);
                if (activo == null)
                {
                    return NotFound("El activo no fue encontrado.");
                }

                _context.Activos.Remove(activo);
                await _context.SaveChangesAsync();

                return RedirectToAction(nameof(Index));
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", "Ocurrió un error inesperado al intentar eliminar el activo.");
                return View("Error", new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier, Message = ex.Message });
            }
        }

        public IActionResult FiltrarActivos(string categoria, string CodigoInventario, string ubicacion, string categoriamaster)
        {
            var activos = _context.Activos.Include(a => a.Usuario)
                                           .Include(a => a.Categoria)
                                           .Include(a => a.Ubicacion)
                                           .AsQueryable();

            // Filtrar por Código de Inventario
            if (!string.IsNullOrEmpty(CodigoInventario))
            {
                activos = activos.Where(a => a.CodigoInventario.Contains(CodigoInventario));
            }

            // Filtrar por Categoría
            if (!string.IsNullOrEmpty(categoria))
            {
                activos = activos.Where(a => a.Categoria.CategoriaNombre.Contains(categoria));
            }

            // Filtrar por Ubicación
            if (!string.IsNullOrEmpty(ubicacion))
            {
                activos = activos.Where(a => a.Ubicacion.UbicacionNombre.Contains(ubicacion));
            }

            // Filtrar por Categoría Maestra
            if (!string.IsNullOrEmpty(categoriamaster))
            {
                var subcategorias = _context.Categoria
                                            .Where(c => c.CategoriaMaster.Nombre == categoriamaster)
                                            .Select(c => c.CategoriaNombre)
                                            .ToList();

                activos = activos.Where(a => subcategorias.Contains(a.Categoria.CategoriaNombre));
            }

            var listaActivos = activos.ToList();

            var model = new ListaActivosViewModel
            {
                ListaActivos = listaActivos
            };

            return PartialView("_ListaActivosPartial", model);
        }

        public IActionResult ObtenerSubcategorias(string categoriaMaster)
        {
            // Asegúrate de que categoriaMaster no esté vacío o nulo
            if (string.IsNullOrEmpty(categoriaMaster))
            {
                return Json(new List<object>()); // Retorna una lista vacía si no hay valor
            }

            var subcategorias = _context.Categoria
                .Where(s => s.CategoriaMaster.Nombre == categoriaMaster)
                .Select(s => new { s.CategoriaNombre })
                .ToList();

            return Json(subcategorias);
        }

        [HttpGet]
        public async Task<IActionResult> OrdenarPorNombre(string sortOrder)
        {
            var activos = _context.Activos
                                  .Include(u => u.Usuario)
                                  .Include(u => u.Ubicacion)
                                  .Include(u => u.Categoria)
                                  .AsQueryable();

            // Ordenar por nombre dependiendo del sortOrder ('asc' o 'desc')
            if (sortOrder == "asc")
            {
                activos = activos.OrderBy(a => a.Usuario.UsuarioNombre);
            }
            else
            {
                activos = activos.OrderByDescending(a => a.Usuario.UsuarioNombre);
            }

            var model = new ListaActivosViewModel
            {
                ListaActivos = await activos.ToListAsync()
            };

            // Devolvemos la vista parcial actualizada con la lista ordenada
            return PartialView("_ListaActivosPartial", model);
        }

        [HttpPost]
        public IActionResult ActualizarFuncionabilidad(int id)
        {
            var activo = _context.Activos.FirstOrDefault(a => a.ActivoId == id);
            if (activo != null)
            {
                activo.Funcionabilidad = !activo.Funcionabilidad; // Cambiar estado
                try
                {
                    _context.SaveChanges();
                    TempData["SuccessMessage"] = "El estado de funcionabilidad del activo se actualizó correctamente.";
                }
                catch (Exception ex)
                {
                    TempData["ErrorMessage"] = "Ocurrió un error al intentar actualizar el activo.";
                }
            }
            else
            {
                TempData["ErrorMessage"] = "Activo no encontrado.";
            }
            return RedirectToAction("Index"); // Redirige a la lista de activos o donde lo necesites
        }

        [HttpPost]
        public IActionResult ReasignarActivo(int id, int usuarioId)
        {
            var activo = _context.Activos.Find(id);
            if (activo == null)
            {
                // Manejo de errores: redirigir a una vista de error o agregar un mensaje
                return NotFound(); // O una vista de error
            }

            activo.UsuarioId = usuarioId;

            try
            {
                _context.SaveChanges();
            }
            catch (Exception ex)
            {
                TempData["ErrorMessage"] = "Ocurrió un error al intentar reasignar el activo. Por favor, inténtalo de nuevo.";
                return RedirectToAction("Index"); 
            }

            return RedirectToAction("Index"); // O la vista que prefieras.
        }
        [HttpPost]
        public IActionResult RelocalizarActivo(int id, int ubicacionId)
        {
            var activo = _context.Activos.FirstOrDefault(a => a.ActivoId == id);
            if (activo != null)
            {
                activo.UbicacionId = ubicacionId;
                try
                {
                    _context.SaveChanges();
                }
                catch (Exception ex)
                {
                    TempData["ErrorMessage"] = "Ocurrió un error al intentar reasignar el activo. Por favor, inténtalo de nuevo.";
                    return RedirectToAction("Index");
                }

                return RedirectToAction("Index");
            }
            return RedirectToAction("Index"); // O la vista que corresponda.
        }

    }
}
